/**
 * @file crypto_config.h
 * @brief CycloneCrypto configuration file
 *
 * @section License
 *
 * Copyright (C) 2010-2016 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneCrypto Open.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.7.2
 **/

#ifndef _CRYPTO_CONFIG_H
#define _CRYPTO_CONFIG_H

//*** <<< Use Configuration Wizard in Context Menu >>> ***

// <o>Trace level
// <i>Set the desired debugging level
// <i>Default: Info
// <0=>Off
// <1=>Fatal
// <2=>Error
// <3=>Warning
// <4=>Info
// <5=>Debug
#define CRYPTO_TRACE_LEVEL 0

// <q>Assembly optimizations
// <i>Enable assembly optimizations for time-critical routines
// <i>Default: Enabled
#define MPI_ASM_SUPPORT 1

// <q>Base64
// <i>Enable Base64 encoding support
// <i>Default: Enabled
#define BASE64_SUPPORT 1

// <q>HMAC
// <i>Enable HMAC support
// <i>Default: Enabled
#define HMAC_SUPPORT 1

// <h>Hash algorithms

//    <q>MD2
//    <i>Enable MD2 hash support
//    <i>Default: Disabled
#define MD2_SUPPORT 0

//    <q>MD4
//    <i>Enable MD4 hash support
//    <i>Default: Disabled
#define MD4_SUPPORT 0

//    <q>MD5
//    <i>Enable MD5 hash support
//    <i>Default: Enabled
#define MD5_SUPPORT 1

//    <q>RIPEMD-128
//    <i>Enable RIPEMD-128 hash support
//    <i>Default: Disabled
#define RIPEMD128_SUPPORT 0

//    <q>RIPEMD-160
//    <i>Enable RIPEMD-160 hash support
//    <i>Default: Disabled
#define RIPEMD160_SUPPORT 0

//    <q>SHA-1
//    <i>Enable SHA-1 hash support
//    <i>Default: Enabled
#define SHA1_SUPPORT 1

//    <q>SHA-224
//    <i>Enable SHA-224 hash support
//    <i>Default: Enabled
#define SHA224_SUPPORT 1

//    <q>SHA-256
//    <i>Enable SHA-256 hash support
//    <i>Default: Enabled
#define SHA256_SUPPORT 1

//    <q>SHA-384
//    <i>Enable SHA-384 hash support
//    <i>Default: Enabled
#define SHA384_SUPPORT 1

//    <q>SHA-512
//    <i>Enable SHA-512 hash support
//    <i>Default: Enabled
#define SHA512_SUPPORT 1

//    <q>SHA-512/224
//    <i>Enable SHA-512/224 hash support
//    <i>Default: Disabled
#define SHA512_224_SUPPORT 0

//    <q>SHA-512/256
//    <i>Enable SHA-512/256 hash support
//    <i>Default: Disabled
#define SHA512_256_SUPPORT 0

//    <q>Tiger
//    <i>Enable Tiger hash support
//    <i>Default: Disabled
#define TIGER_SUPPORT 0

//    <q>Whirlpool
//    <i>Enable Whirlpool hash support
//    <i>Default: Disabled
#define WHIRLPOOL_SUPPORT 0

// </h>
// <h>Encryption algorithms

//    <q>RC4
//    <i>Enable RC4 cipher support
//    <i>Default: Enabled
#define RC4_SUPPORT 1

//    <q>RC6
//    <i>Enable RC6 cipher support
//    <i>Default: Disabled
#define RC6_SUPPORT 0

//    <q>IDEA
//    <i>Enable IDEA cipher support
//    <i>Default: Enabled
#define IDEA_SUPPORT 1

//    <q>DES
//    <i>Enable DES cipher support
//    <i>Default: Enabled
#define DES_SUPPORT 1

//    <q>Triple DES
//    <i>Enable Triple DES cipher support
//    <i>Default: Enabled
#define DES3_SUPPORT 1

//    <q>AES
//    <i>Enable AES cipher support
//    <i>Default: Enabled
#define AES_SUPPORT 1

//    <q>Camellia
//    <i>Enable Camellia cipher support
//    <i>Default: Enabled
#define CAMELLIA_SUPPORT 1

//    <q>SEED
//    <i>Enable SEED cipher support
//    <i>Default: Disabled
#define SEED_SUPPORT 0

//    <q>ARIA
//    <i>Enable ARIA cipher support
//    <i>Default: Disabled
#define ARIA_SUPPORT 0

// </h>
// <h>Cipher modes

//    <q>ECB mode
//    <i>Enable ECB mode support
//    <i>Default: Enabled
#define ECB_SUPPORT 1

//    <q>CBC mode
//    <i>Enable CBC mode support
//    <i>Default: Enabled
#define CBC_SUPPORT 1

//    <q>CFB mode
//    <i>Enable CFB mode support
//    <i>Default: Enabled
#define CFB_SUPPORT 1

//    <q>OFB mode
//    <i>Enable OFB mode support
//    <i>Default: Enabled
#define OFB_SUPPORT 1

//    <q>CTR mode
//    <i>Enable CTR mode support
//    <i>Default: Enabled
#define CTR_SUPPORT 1

//    <q>CCM mode
//    <i>Enable CCM mode support
//    <i>Default: Enabled
#define CCM_SUPPORT 1

//    <q>GCM mode
//    <i>Enable GCM mode support
//    <i>Default: Enabled
#define GCM_SUPPORT 1

// </h>

#endif
