/**
 * @file tls_config.h
 * @brief CycloneSSL configuration file
 *
 * @section License
 *
 * Copyright (C) 2010-2016 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneSSL Open.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.7.2
 **/

#ifndef _TLS_CONFIG_H
#define _TLS_CONFIG_H

//*** <<< Use Configuration Wizard in Context Menu >>> ***

// <o>Trace level
// <i>Set the desired debugging level
// <i>Default: Info
// <0=>Off
// <1=>Fatal
// <2=>Error
// <3=>Warning
// <4=>Info
// <5=>Debug
#define TLS_TRACE_LEVEL 4

// <q>SSL/TLS support
// <i>Enable SSL/TLS support
// <i>Default: Enabled
#define TLS_SUPPORT 1

// <q>Client mode of operation
// <i>Enable client mode of operation
// <i>Default: Enabled
#define TLS_CLIENT_SUPPORT 1

// <q>Server mode of operation
// <i>Enable server mode of operation
// <i>Default: Enabled
#define TLS_SERVER_SUPPORT 1

// <o>Minimum SSL/TLS version
// <i>Minimum version that can be negotiated
// <i>Default: SSL 3.0
// <0x0300=>SSL 3.0
// <0x0301=>TLS 1.0
// <0x0302=>TLS 1.1
// <0x0303=>TLS 1.2
#define TLS_MIN_VERSION 0x0300

// <o>Maximum SSL/TLS version
// <i>Maximum version that can be negotiated
// <i>Default: TLS 1.2
// <0x0300=>SSL 3.0
// <0x0301=>TLS 1.0
// <0x0302=>TLS 1.1
// <0x0303=>TLS 1.2
#define TLS_MAX_VERSION 0x0303

//Use BSD socket API
#define TLS_BSD_SOCKET_SUPPORT ENABLED

// <q>Session resumption
// <i>Enable session resumption mechanism
// <i>Default: Enabled
#define TLS_SESSION_RESUME_SUPPORT 1

// <o>Session cache entry lifetime
// <i>Lifetime of session cache entries (in seconds)
// <i>Default: 3600
// <10-86400>
// <#*1000>
#define TLS_SESSION_CACHE_LIFETIME 3600000

// <q>SNI extension
// <i>Enable SNI (Server Name Indication) extension
// <i>Default: Enabled
#define TLS_SNI_SUPPORT ENABLED

// <o>Maximum number of certificates
// <i>Maximum number of certificates the end entity can load
// <i>Default: 3
// <1-8>
#define TLS_MAX_CERTIFICATES 3

// <o>Maximum message length
// <i>Maximum message length that can be handled by the higher-level protocol
// <i>Default: 16384
// <1024-65536>
#define TLS_MAX_PROTOCOL_DATA_LENGTH 16384

// <h>Key Exchange algorithms

//    <q>RSA key exchange
//    <i>Enable RSA key exchange support
//    <i>Default: Enabled
#define TLS_RSA_SUPPORT 1

//    <q>DHE_RSA key exchange
//    <i>Enable DHE_RSA key exchange support
//    <i>Default: Disabled
#define TLS_DHE_RSA_SUPPORT 0

//    <q>DHE_DSS key exchange
//    <i>Enable DHE_DSS key exchange support
//    <i>Default: Disabled
#define TLS_DHE_DSS_SUPPORT 0

//    <q>DH_anon key exchange
//    <i>Enable DH_anon key exchange support
//    <i>Default: Disabled
#define TLS_DH_ANON_SUPPORT 0

//    <q>ECDHE_RSA key exchange
//    <i>Enable ECDHE_RSA key exchange support
//    <i>Default: Enabled
#define TLS_ECDHE_RSA_SUPPORT 1

//    <q>ECDHE_ECDSA key exchange
//    <i>Enable ECDHE_ECDSA key exchange support
//    <i>Default: Disabled
#define TLS_ECDHE_ECDSA_SUPPORT 0

//    <q>ECDH_anon key exchange support
//    <i>Enable ECDH_anon key exchange support
//    <i>Default: Disabled
#define TLS_ECDH_ANON_SUPPORT 0

// </h>
// <h>Signature algorithms

//    <q>RSA signature
//    <i>Enable RSA signature capability
//    <i>Default: Enabled
#define TLS_RSA_SIGN_SUPPORT 1

//    <q>DSA signature
//    <i>Enable DSA signature capability
//    <i>Default: Disabled
#define TLS_DSA_SIGN_SUPPORT 0

//    <q>ECDSA signature
//    <i>Enable ECDSA signature capability
//    <i>Default: Enabled
#define TLS_ECDSA_SIGN_SUPPORT 1

// </h>
// <h>Cipher types

//    <q>Stream cipher
//    <i>Enable stream cipher support
//    <i>Default: Enabled
#define TLS_STREAM_CIPHER_SUPPORT 1

//    <q>CBC block cipher
//    <i>Enable CBC block cipher support
//    <i>Default: Enabled
#define TLS_CBC_CIPHER_SUPPORT 1

//    <q>CCM mode
//    <i>Enable CCM mode support
//    <i>Default: Enabled
#define TLS_CCM_CIPHER_SUPPORT 0

//    <q>GCM mode
//    <i>Enable GCM mode support
//    <i>Default: Enabled
#define TLS_GCM_CIPHER_SUPPORT 0

// </h>
// <h>Encryption algorithms

//    <q>RC4
//    <i>Enable RC4 cipher support
//    <i>Default: Enabled
#define TLS_RC4_SUPPORT 1

//    <q>IDEA
//    <i>Enable IDEA cipher support
//    <i>Default: Disabled
#define TLS_IDEA_SUPPORT 0

//    <q>DES
//    <i>Enable DES cipher support
//    <i>Default: Disabled
#define TLS_DES_SUPPORT 0

//    <q>Triple DES
//    <i>Enable Triple DES cipher support
//    <i>Default: Enabled
#define TLS_3DES_SUPPORT 1

//    <q>AES
//    <i>Enable AES cipher support
//    <i>Default: Enabled
#define TLS_AES_SUPPORT 1

//    <q>Camellia
//    <i>Enable Camellia cipher support
//    <i>Default: Enabled
#define TLS_CAMELLIA_SUPPORT 1

//    <q>SEED
//    <i>Enable SEED cipher support
//    <i>Default: Disabled
#define TLS_SEED_SUPPORT 0

//    <q>ARIA
//    <i>Enable ARIA cipher support
//    <i>Default: Disabled
#define TLS_ARIA_SUPPORT 0

// </h>
// <h>Hash algorithms

//    <q>MD5
//    <i>Enable MD5 hash support
//    <i>Default: Enabled
#define TLS_MD5_SUPPORT 1

//    <q>SHA-1
//    <i>Enable SHA-1 hash support
//    <i>Default: Enabled
#define TLS_SHA1_SUPPORT 1

//    <q>SHA-224
//    <i>Enable SHA-224 hash support
//    <i>Default: Enabled
#define TLS_SHA224_SUPPORT 1

//    <q>SHA-256
//    <i>Enable SHA-256 hash support
//    <i>Default: Enabled
#define TLS_SHA256_SUPPORT 1

//    <q>SHA-384
//    <i>Enable SHA-384 hash support
//    <i>Default: Enabled
#define TLS_SHA384_SUPPORT 1

//    <q>SHA-512
//    <i>Enable SHA-512 hash support
//    <i>Default: Enabled
#define TLS_SHA512_SUPPORT 1

// </h>
// <h>Elliptic curves

//    <q>secp160k1
//    <i>Enable secp160k1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP160K1_SUPPORT 0

//    <q>secp160r1
//    <i>Enable secp160r1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP160R1_SUPPORT 0

//    <q>secp160r2
//    <i>Enable secp160r2 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP160R2_SUPPORT 0

//    <q>secp192k1
//    <i>Enable secp192k1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP192K1_SUPPORT 0

//    <q>secp192r1
//    <i>Enable secp192r1 elliptic curve support
//    <i>Default: Enabled
#define TLS_SECP192R1_SUPPORT 1

//    <q>secp224k1
//    <i>Enable secp224k1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP224K1_SUPPORT 0

//    <q>secp224r1
//    <i>Enable secp224r1 elliptic curve support
//    <i>Default: Enabled
#define TLS_SECP224R1_SUPPORT 1

//    <q>secp256k1
//    <i>Enable secp256k1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP256K1_SUPPORT 0

//    <q>secp256r1
//    <i>Enable secp256r1 elliptic curve support
//    <i>Default: Enabled
#define TLS_SECP256R1_SUPPORT 1

//    <q>secp384r1
//    <i>Enable secp384r1 elliptic curve support
//    <i>Default: Enabled
#define TLS_SECP384R1_SUPPORT 1

//    <q>secp521r1
//    <i>Enable secp521r1 elliptic curve support
//    <i>Default: Disabled
#define TLS_SECP521R1_SUPPORT 0

//    <q>brainpoolP256r1
//    <i>Enable brainpoolP256r1 elliptic curve support
//    <i>Default: Disabled
#define TLS_BRAINPOOLP256R1_SUPPORT 0

//    <q>brainpoolP384r1
//    <i>Enable brainpoolP384r1 elliptic curve support
//    <i>Default: Disabled
#define TLS_BRAINPOOLP384R1_SUPPORT 0

//    <q>brainpoolP512r1
//    <i>Enable brainpoolP512r1 elliptic curve support
//    <i>Default: Disabled
#define TLS_BRAINPOOLP512R1_SUPPORT 0

// </h>

#endif
